/**
 * $Id: ConnectExceptionAspect.aj,v 1.1 2011/12/22 17:35:49 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * (c)reated 21.01.2009 by oliver (ob@oasd.de)
 */
package patterntesting.exception.net;

import java.io.IOException;
import java.net.*;

import org.aspectj.lang.annotation.SuppressAjWarnings;
import org.slf4j.*;

/**
 * If you get a ConnectException you do not always know why. Which host or
 * InetAdress was it? Or which port? These informations are missing in the
 * thrown ConnectException. Until now...
 *
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 0.9.2
 */
public aspect ConnectExceptionAspect {

    private static final Logger log = LoggerFactory.getLogger(ConnectExceptionAspect.class);

    /**
     * If a socket is created you may get a ConnectException with
     * "connection refused". What you don't see is the host and the port
     * in the exception. So this is now provided by this advice.
     */
    @SuppressAjWarnings({"adviceDidNotMatch"})
    Socket around(final String host, final int port) throws IOException :
            call(public Socket.new(String, int, ..) throws IOException)
            && args(host, port) {
        try {
            return proceed(host, port);
        } catch (ConnectException ce) {
            String msg = ce.getMessage();
            if (log.isTraceEnabled()) {
                log.trace("improving message of'" + ce + "'...");
            }
            if (msg.equals("Connection refused")) {
                msg = "Connection to " + host + ":" + port + " refused";
            } else {
                msg += " (" + host + ":" + port + ")";
            }
            throw new ConnectException(msg);
        }
    }

}
